/* Theme agnostic or base-layer CSS styling */
@import url("resource:///org/gnome/dazzle/themes/shared/shared-graphs.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-menus.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-panels.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-pathbar.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-pillbox.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-preferences.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-progressbutton.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-stacklist.css");
@import url("resource:///org/gnome/dazzle/themes/shared/shared-suggestions.css");
